"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCalendarDate = exports.Date = exports.UNIT = void 0;
var luxon_1 = require("luxon");
var number_1 = require("../utilities/number");
var UNIT;
(function (UNIT) {
    UNIT["YEAR"] = "year";
    UNIT["MONTH"] = "month";
    UNIT["WEEK"] = "week";
    UNIT["DAY"] = "day";
})(UNIT = exports.UNIT || (exports.UNIT = {}));
var Date = /** @class */ (function () {
    /**
     * Constructor
     * @param params year, month, day
     */
    function Date(params) {
        Object.defineProperty(this, "dateTime", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        this.dateTime = params
            ? luxon_1.DateTime.local(params.year, params.month, params.day)
            : luxon_1.DateTime.now();
    }
    /**
     * Constructor
     * @param date time string ex) "2022-03-14"
     * @returns Date
     */
    Object.defineProperty(Date, "string", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (date) {
            var split = date.split("-");
            var year = +split[0];
            var month = Math.min(+split[1], 12);
            var day = Math.min(+split[2], luxon_1.DateTime.local(year, month).endOf("month").day);
            return new Date({ year: year, month: month, day: day });
        }
    });
    /**
     * @returns luxon datetime
     */
    Object.defineProperty(Date.prototype, "getDateTime", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime;
        }
    });
    /**
     * @returns year of date
     */
    Object.defineProperty(Date.prototype, "getYear", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime.year;
        }
    });
    /**
     * @returns month of date
     */
    Object.defineProperty(Date.prototype, "getMonth", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime.month;
        }
    });
    /**
     * @returns weekday of date
     */
    Object.defineProperty(Date.prototype, "getWeekday", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime.weekday;
        }
    });
    /**
     * @returns weeknumber of date
     */
    Object.defineProperty(Date.prototype, "getWeekNumber", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime.weekNumber;
        }
    });
    /**
     * @returns day of date
     */
    Object.defineProperty(Date.prototype, "getDay", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            return this.dateTime.day;
        }
    });
    /**
     * @param unit year, month, week, day
     * @returns start of the unit
     */
    Object.defineProperty(Date.prototype, "getStart", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (unit) {
            return this.getDateTime().startOf(unit);
        }
    });
    /**
     * @param unit year, month, week, day
     * @returns end of the unit
     */
    Object.defineProperty(Date.prototype, "getEnd", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (unit) {
            return this.getDateTime().endOf(unit);
        }
    });
    /**
     * @param year year to set
     * @param month month to set
     * @param day day to set
     */
    Object.defineProperty(Date.prototype, "set", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (_a) {
            var year = _a.year, month = _a.month, day = _a.day;
            this.dateTime = this.dateTime.set({ year: year, month: month, day: day });
        }
    });
    /**
     * @param year year to add
     * @param month month to add
     * @param day day to add
     */
    Object.defineProperty(Date.prototype, "plus", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (_a) {
            var year = _a.year, month = _a.month, day = _a.day;
            this.dateTime = this.dateTime.plus({ year: year, month: month, day: day });
        }
    });
    /**
     * @param date date to compare
     * @returns is euqal date
     */
    Object.defineProperty(Date.prototype, "isEqual", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function (date) {
            return this.getDateTime().equals(date.getDateTime());
        }
    });
    /**
     * @returns date string ex) "2022-03-14"
     */
    Object.defineProperty(Date.prototype, "getString", {
        enumerable: false,
        configurable: true,
        writable: true,
        value: function () {
            var year = (0, number_1.fillZero)(this.getYear(), 4);
            var month = (0, number_1.fillZero)(this.getMonth());
            var day = (0, number_1.fillZero)(this.getDay());
            return "".concat(year, "-").concat(month, "-").concat(day);
        }
    });
    return Date;
}());
exports.Date = Date;
var getCalendarDate = function (_a) {
    var date = _a.date, _b = _a.mondayFirst, mondayFirst = _b === void 0 ? true : _b;
    var startDate = mondayFirst
        ? getNearestPrevMonday(date.getDateTime().startOf("month"))
        : getNearestPrevSunday(date.getDateTime().startOf("month"));
    var endDate = mondayFirst
        ? getNearestNextSunday(date.getDateTime().endOf("month"))
        : getNearestNextSaturday(date.getDateTime().endOf("month"));
    var calendarDate = [];
    for (var i = startDate; i <= endDate; i = i.plus({ days: 1 })) {
        var year = i.year, month = i.month, day = i.day;
        calendarDate.push(new Date({ year: year, month: month, day: day }));
    }
    return calendarDate;
};
exports.getCalendarDate = getCalendarDate;
var getNearestPrevMonday = function (dateTime) {
    var dist = 1 - dateTime.weekday;
    return dateTime.plus({ days: dist });
};
var getNearestNextSunday = function (dateTime) {
    var dist = 7 - dateTime.weekday;
    return dateTime.plus({ days: dist });
};
var getNearestPrevSunday = function (dateTime) {
    var dist = 0 - dateTime.weekday;
    return dateTime.plus({ days: dist });
};
var getNearestNextSaturday = function (dateTime) {
    var dist = 6 - dateTime.weekday;
    return dateTime.plus({ days: dist });
};
